@@ -172,7 +172,7 @@ static gint gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpa
 static gchar *gnome_print_pdf_get_date (void);
 
 static gint  gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, PdfGraphicMode mode);
-static gint  gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...);
+static gint  gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...) G_GNUC_PRINTF (2, 3);
 
 static GnomePrintContextClass *parent_class;
 
@@ -576,7 +576,7 @@ gnome_print_pdf_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 {
 	GnomePrintPdfPage *page;
 	GnomePrintPdf *pdf;
-	gint hex_size, row, object_num, ret = 0, mask_object_num;
+	gint hex_size, row, object_num, ret = 0, mask_object_num = 0;
 	guchar *hex;
 	gint length_object_num, len = 0;
 
@@ -719,16 +719,18 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 {
 	static gdouble id[] = {1.0, 0.0, 0.0, 1.0, 0.0, 0.0};
 
+	GnomePrintPdf	  *pdf = GNOME_PRINT_PDF (pc);
 	GnomePrintPdfPage *page;
-	ArtPoint pt;
-	GnomePrintPdf *pdf;
 	GnomePosGlyphList *pgl;
-	gint ret, s, master_glyph, glyph_page, x_adjust;
-	gint glyph; /* Glyph in subfont */
-	gdouble dx, dy, prev;
-	gdouble Tm [6], scale [6];
-
-	pdf = GNOME_PRINT_PDF (pc);
+	GnomePosGlyph const *glyph;
+	ArtPoint advance;
+	char	 octal [ 1 /* header char  \\ */ +
+			11 /* 32 bit int in octal */ +
+			 1 /* trailing null */];
+	gint	 ret, s, master_glyph, glyph_page, code;
+	gint	 x_adjust, y_adjust, cur_y_adjust = 0;
+	gdouble	 Tm [6], scale [6], base_y, prev_x = 0;
+	enum { OUTSIDE, IN_ARRAY, IN_GLYPHS } state;
 
 	gnome_print_pdf_graphic_mode_set (pdf, PDF_GRAPHIC_MODE_TEXT);
 
@@ -737,9 +739,6 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 
 	pgl = gnome_pgl_from_gl (gl, id, GNOME_PGL_RENDER_DEFAULT);
 
-	dx = ctm[4];
-	dy = ctm[5];
-
 	for (s = 0; s < pgl->num_strings; s++) {
 		GnomePosString *ps;
 		GnomeFont *font;
@@ -755,6 +754,8 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 						      ((ps->color >>  8) & 0xff) / 255.0);
 
 		art_affine_scale (scale, font->size, font->size);
+		scale[4]	  = pgl->glyphs[ps->start].x;
+		scale[5] = base_y = pgl->glyphs[ps->start].y;
 		art_affine_multiply (Tm, scale, ctm);
 
 		gnome_print_pdf_page_print_double (pdf, "%.2f", Tm [0]);
@@ -765,88 +766,82 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 		gnome_print_pdf_page_write (pdf, " ");
 		gnome_print_pdf_page_print_double (pdf, "%.2f", Tm [3]);
 		gnome_print_pdf_page_write (pdf, " ");
-		gnome_print_pdf_page_print_double (pdf, "%.2f", dx + pgl->glyphs[ps->start].x);
+		gnome_print_pdf_page_print_double (pdf, "%.2f", Tm [4]);
 		gnome_print_pdf_page_write (pdf, " ");
-		gnome_print_pdf_page_print_double (pdf, "%.2f", dy + pgl->glyphs[ps->start].y);
+		gnome_print_pdf_page_print_double (pdf, "%.2f", Tm [5]);
 		gnome_print_pdf_page_write (pdf, " Tm" EOL);
 
 		/* Build string */
-		if (NEEDED_SUBSETTING (font)) {
+		state = OUTSIDE;
 			for (i = ps->start; i < ps->start + ps->length; i++) {
-				/* Glyph position in parent font */
-				master_glyph = (gint)(pgl->glyphs[i].glyph);
-				/* Pages of 256 glyphs, with leading null */
-				glyph_page = master_glyph / 255;
+			glyph	     = pgl->glyphs + i;
+			master_glyph = glyph->glyph;
 
-				if (i == ps->start) {		/* Initially */
-					current_page = glyph_page;
-					gnome_print_pdf_set_font_real (pdf, font, TRUE, glyph_page);
-					ret = gnome_print_pdf_page_write (pdf, "[(");
-				} else {
-					gnome_font_face_get_glyph_stdadvance (font->face, pgl->glyphs[i-1].glyph, &pt);
-					x_adjust =
-						(int)  (pt.x + .5) -
-						(int) ((1000 *(pgl->glyphs[i].x - prev) / Tm[0]) + .5);
+			if (i != ps->start) {
+				state = IN_GLYPHS;
+				x_adjust = (int)  (advance.x + .5) -
+					   (int) ((1000. *(glyph->x - prev_x) / font->size) + .5);
+				y_adjust = (int)  (advance.y + .5) -
+					   (int) ((1000. *(glyph->y - base_y) / font->size) + .5);
 #if 0
-					fprintf (stderr, "advance [%d] = %4.0f\tvs a delta of\t%g\n", i,
-						 pt.x, 1000 *(pgl->glyphs[i].x - prev) / Tm[0]);
+				fprintf (stderr, "advance [%d].x = %4.0f\tvs a delta of\t%g\n", i,
+					 advance.x, 1000. *(glyph->x - prev_x) / font->scale);
+				fprintf (stderr, "advance [%d].y = %4.0f\tvs a delta of\t%g\n", i,
+					 advance.y, 1000. *(glyph->y - base_y) / font->scale);
 #endif
 					if (x_adjust != 0) {
-						if (current_page != glyph_page) { /* There's a ttf subfont to be set*/
-							current_page = glyph_page;
-							gnome_print_pdf_page_fprintf (pdf, ") %d] TJ" EOL, x_adjust);
-							gnome_print_pdf_set_font_real (pdf, font, TRUE, glyph_page);
-							gnome_print_pdf_page_write (pdf, "[(");
-						} else
-							gnome_print_pdf_page_fprintf (pdf, ") %d (", x_adjust);
+					gnome_print_pdf_page_fprintf (pdf, ") %d" EOL, x_adjust);
+					state = IN_ARRAY;
+				}
 
-					} else if (current_page != glyph_page) { /* There's a ttf subfont to be set*/
-						current_page = glyph_page;
+				if (cur_y_adjust != y_adjust) {
+					if (state == IN_GLYPHS)
+						gnome_print_pdf_page_write (pdf, ")");
+					gnome_print_pdf_page_fprintf (pdf, " %d] TJ " EOL,
+						(cur_y_adjust = y_adjust));
+					state = OUTSIDE;
+				}
+				if (NEEDED_SUBSETTING (font)) {
+					glyph_page = master_glyph / 255;
+					if (glyph_page != current_page) {
+						if (state == IN_GLYPHS)
 						gnome_print_pdf_page_write (pdf, ")] TJ" EOL);
-						gnome_print_pdf_set_font_real (pdf, font, TRUE, glyph_page);
-						gnome_print_pdf_page_write (pdf, "[(");
+						else if (state == IN_ARRAY)
+							gnome_print_pdf_page_write (pdf, "] TJ" EOL);
+						state = OUTSIDE;
+						gnome_print_pdf_set_font_real (pdf, font, TRUE,
+							(current_page = glyph_page));
 					}
 				}
-				prev = pgl->glyphs[i].x;
+			} else if (NEEDED_SUBSETTING (font)) {
+				current_page   = master_glyph / 255;
+				gnome_print_pdf_set_font_real (pdf, font, TRUE, current_page);
+			} else
+				gnome_print_pdf_set_font_real (pdf, font, FALSE, 0);
 
+			if (NEEDED_SUBSETTING (font)) {
 				gnome_font_face_pso_mark_glyph (pdf->selected_font->pso, master_glyph);
+				/* For each sub ttf font the first glyph is
+				 * null and 255th one is a valid glyph, also 1
+				 * needed to be added to each subglyph  for
+				 * taking into account first null glyph added
+				 * to every subfont */
+				code = master_glyph ? master_glyph % 255 + 1 : master_glyph;
+			} else
+				code = gnome_print_pdf_assign_code_to_glyph (pdf, master_glyph);
 
-				/* For each sub ttf font the first glyph is  */
-				/* null and 255th one is a valid glyph, also */
-				/* 1 needed to be added to each subglyph     */
-				/* for taking into account first null glyph  */
-				/* added to every subfont	             */
-
-				glyph = master_glyph ? master_glyph % 255 + 1 : master_glyph;
-				ret = gnome_print_pdf_page_fprintf (pdf, "\\%o", glyph);
-
-				g_return_val_if_fail (ret >= 0, ret);
+			if (state < IN_GLYPHS) {
+				ret = gnome_print_pdf_page_write (pdf, (state == OUTSIDE) ? "[(" : "(");
+				state = IN_GLYPHS;
 			}
-		} else {
-			gnome_print_pdf_set_font_real (pdf, font, FALSE, 0);
 
-			ret = gnome_print_pdf_page_write (pdf, "[(");
-			for (i = ps->start; i < ps->start + ps->length; i++) {
-				if (i != ps->start) {
-					gnome_font_face_get_glyph_stdadvance (font->face, glyph, &pt);
-					x_adjust =
-						(int)  (pt.x + .5) -
-						(int) ((1000 *(pgl->glyphs[i].x - prev) / Tm[0]) + .5);
-#if 0
-					fprintf (stderr, "advance [%d] = %4.0f\tvs a delta of\t%g\n", i,
-						 pt.x, 1000 *(pgl->glyphs[i].x - prev) / Tm[0]);
-#endif
-					if (x_adjust != 0)
-						gnome_print_pdf_page_fprintf (pdf, ") %d (", x_adjust);
-				}
-				prev = pgl->glyphs[i].x;
-				glyph = pgl->glyphs[i].glyph;
-				ret = gnome_print_pdf_page_fprintf (pdf, "\\%o",
-					gnome_print_pdf_assign_code_to_glyph (pdf, glyph));
+			g_snprintf (octal, sizeof(octal), "\\%o", code);
+			ret = gnome_print_pdf_page_write (pdf, octal);
 				g_return_val_if_fail (ret >= 0, ret);
-			}
+
+			gnome_font_face_get_glyph_stdadvance (font->face, master_glyph, &advance);
+			prev_x = glyph->x;
 		}
-
 		ret = gnome_print_pdf_page_write (pdf, ")] TJ" EOL);
 	}
 
@@ -1426,8 +1421,7 @@ gnome_print_pdf_set_line (GnomePrintPdf *pdf)
 					   gp_gc_get_linewidth (ctx->gc));
 	gnome_print_pdf_page_fprintf (pdf, " w %d J %d j ",
 				      gp_gc_get_linecap (ctx->gc),
-				      gp_gc_get_linejoin (ctx->gc),
-				      gp_gc_get_miterlimit (ctx->gc));
+				      gp_gc_get_linejoin (ctx->gc));
 	gnome_print_pdf_page_print_double (pdf, "%.2f",
 					   gp_gc_get_miterlimit (ctx->gc));
 	gnome_print_pdf_page_write (pdf, " M" EOL);
@@ -1548,7 +1542,7 @@ gnome_print_pdf_font_print_widths (GnomePrintPdf *pdf, GnomePrintPdfFont *font)
 			col = 0;
 		}
 		col += gnome_print_pdf_print_double (pdf, "%.1f", point.x);
-		col += gnome_print_pdf_fprintf (pdf, " ");
+		gnome_print_pdf_print_sized (pdf, " ", 1);
 		i++;
 	}
 
@@ -1710,9 +1704,9 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 	g_object_get (G_OBJECT (face), "XHeight", &val, NULL);
 	xheight = (gint) val;
 	g_object_get (G_OBJECT (face), "FontBBox", &rect, NULL);
-	g_free (rect);
 	art_drect_copy (&new_rect, rect);
 	my_fix_drect (&new_rect);
+	g_free (rect);
 
 	object_number_descriptor = gnome_print_pdf_object_new (pdf);
 	*object_number_descriptor_ret = object_number_descriptor;
@@ -1763,7 +1757,8 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
  */
 
 static guchar *
-gnome_print_pdf_get_subfont_name (guchar *basefont, guint unique_subfont_id) {
+gnome_print_pdf_get_subfont_name (guchar *basefont, guint unique_subfont_id)
+{
 	int i;
 	guchar *full_basefont;
 
@@ -1783,7 +1778,8 @@ gnome_print_pdf_get_subfont_name (guchar *basefont, guint unique_subfont_id) {
 }
 
 void
-gnome_print_embed_pdf_font (GnomePrintPdf *pdf, GnomePrintPdfFont *font) {
+gnome_print_embed_pdf_font (GnomePrintPdf *pdf, GnomePrintPdfFont *font)
+{
 	gint object_number_descriptor = 0;
 	guchar *basefont_name;
 	gboolean plan_b = FALSE; /* TRUE if we could not embed the font */
@@ -2085,8 +2081,8 @@ gnome_print_pdf_print_double (GnomePrintPdf *pdf, const gchar *format, gdouble x
  	gchar text [G_ASCII_DTOSTR_BUF_SIZE];
 	gint len;
 
-	len = strlen (text);
 	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+	len = strlen (text);
 	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, text, len);
 	pdf->offset += len;
 	return len;
@@ -2098,7 +2094,6 @@ gnome_print_pdf_print_sized (GnomePrintPdf *pdf, const char *content, gint len)
 {
 	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, content, len);
 	pdf->offset += len;
-
 	return len;
 }
 
